/*
 *  GUI/SearchFolderDialog.cs
 *  Copyright (C) 2006-2007 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Collections;
using Gtk;
using Glade;
using Mono.Unix;
using Incollector.Engine;
using Incollector.GUI.Widgets;
using Incollector.Searching;


namespace Incollector.GUI {
	
	/*
	 * SearchFolderDialog class
	 */
	
	public class SearchFolderDialog {
		private Glade.XML     gui;
		public  Gtk.Dialog    dialog;
		
		[Glade.Widget] protected Gtk.Notebook MainNotebook;
		[Glade.Widget] protected Gtk.Table    ConditionalsTable;
		[Glade.Widget] protected Gtk.ComboBox ShowEntries;
		[Glade.Widget] protected Gtk.Entry    SearchName;
		[Glade.Widget] protected Gtk.Button   OkBtn;
		[Glade.Widget] protected Gtk.Frame    ConditionalsFrame;
		
		protected ArrayList Conditionals;
		
		public string Name {
			get { return SearchName.Text;  }
			set { SearchName.Text = value; }
		}
		
		public ArrayList QueriesList {
			get {
				ArrayList rtn = new ArrayList ();
				foreach (Conditional cnd in Conditionals)
					rtn.Add (cnd.query);
				
				return rtn;
			}
			
			set {
				while (Conditionals.Count > 0)
					RemoveConditional (Conditionals [0] as Conditional);
				
				foreach (Query query in value)
					AddConditional (new Conditional (query));
			}
		}
		
		public Searching.LogicalOperator ShowingEntriesOperator {
			get {
				if (ShowEntries.Active == 0)
					return Searching.LogicalOperator.And;
				else
					return Searching.LogicalOperator.Or;
			}
			
			set {
				if (value == LogicalOperator.And)
					ShowEntries.Active = 0;
				else
					ShowEntries.Active = 1;
			}
		}
		
		protected enum MainNotebookEnum {
			Info              = 0,
			ConditionalsTable = 1
		};
		
		
		protected class Conditional {
			public    Gtk.ComboBox QueryType;
			public    Gtk.HBox     QueryWidget;
			public    Gtk.Button   RemoveButton;
			public    Query        query;
			public    Query        LoadedQuery;
			protected int          CurrentQueryType;
			
			
			public Conditional ()
			{
				Create ();
			}
			
			
			public Conditional (Query query)
			{
				this.LoadedQuery = query;
				Create ();
			}
			
			
			public void Create ()
			{
				QueryWidget = new Gtk.HBox ();
				QueryWidget.Show ();
				
				
				QueryType = new Gtk.ComboBox (new string [] {});
				QueryType.Changed += new EventHandler (on_QueryType_changed);
				
				int Current = 0, i = 0;
				foreach (Query query in Searching.SearchFolders.QueriesList) {
					if ((this.LoadedQuery != null) && (query.ID == this.LoadedQuery.ID))
						Current = i;
					
					QueryType.AppendText (query.Name);
					i++;
				}
				
				if (Searching.SearchFolders.QueriesList.Count > 0) {
					QueryType.Active = Current;
					CurrentQueryType = Current;
				}
				
				QueryType.Show ();
				
				
				RemoveButton = new Gtk.Button ();
				RemoveButton.Label = Catalog.GetString ("Remove");
				RemoveButton.Image = new Gtk.Image (Gtk.Stock.Remove, Gtk.IconSize.Menu);
				RemoveButton.Show ();
			}
			
			
			protected void on_QueryType_changed (object o, EventArgs args)
			{
				if ((query == null) && (LoadedQuery != null)) {
					query = LoadedQuery;
					QueryWidget.Add (query.QueryWidget);
					QueryWidget.ShowAll ();
				} else {
					if (QueryWidget.Children.Length > 0)
						QueryWidget.Remove (query.QueryWidget);
					
					query = (Searching.SearchFolders.QueriesList [QueryType.Active] as Query).New ();
					QueryWidget.Add (query.QueryWidget);
					QueryWidget.ShowAll ();
				}
			}
		}
		
		
		public SearchFolderDialog (Gtk.Window parent)
		{
			CreateSearchFolderDialog (parent);
		}
		
		
		public SearchFolderDialog (Gtk.Window parent, SearchFolder folder)
		{
			CreateSearchFolderDialog (parent);
			
			ArrayList qrs = new ArrayList ();
			foreach (Query query in new ArrayList (folder.Queries))
				qrs.Add (query.Clone ());
			
			this.QueriesList = qrs;
			this.Name = folder.Name;
			this.ShowingEntriesOperator = folder.SearchOperator;
			
			OkBtn.Label = Catalog.GetString ("_OK");
			OkBtn.Image = new Gtk.Image (Gtk.Stock.Ok, Gtk.IconSize.Button);
		}
		
		
		private void CreateSearchFolderDialog (Gtk.Window parent)
		{
			gui = Glade.XML.FromAssembly ("gui.glade", "SearchFolderDialog", null);
			gui.Autoconnect (this);
			dialog = (Gtk.Dialog) gui.GetWidget ("SearchFolderDialog");
			dialog.TransientFor = parent;
			dialog.VBox.Spacing = 0;
			
			ShowEntries.Active = 0;
			Colors.Modify (Colors.WHITE, "Bg", gui);
			
			Conditionals = new ArrayList ();
			
			dialog.Show ();
		}
		
		
		protected void on_SearchFolderDialog_destroy (object o, EventArgs args)
		{
			foreach (Conditional cnd in Conditionals)
				cnd.QueryWidget.Remove (cnd.query.QueryWidget);
		}
		
		
		protected void on_SearchName_changed (object o, EventArgs args)
		{
			OkBtn.Sensitive = (o as Gtk.Entry).Text != "";
		}
		
		
		protected void on_AddConditionalBtn_clicked (object o, EventArgs args)
		{
			AddConditional (new Conditional ());
		}
		
		
		protected void AddConditional (Conditional ctl)
		{
			ctl.RemoveButton.Clicked += new EventHandler (on_RemoveButton_clicked);
			
			
			MainNotebook.Page = (int) MainNotebookEnum.ConditionalsTable;
			
			Conditionals.Add (ctl);
			uint LastConditional = (uint) Conditionals.Count;
			
			ConditionalsTable.Attach (ctl.QueryType, 0, 1, LastConditional - 1, LastConditional, AttachOptions.Shrink, AttachOptions.Fill, 3, 3);
			ConditionalsTable.Attach (ctl.QueryWidget, 1, 2, LastConditional - 1, LastConditional, AttachOptions.Expand | AttachOptions.Fill, AttachOptions.Fill, 3, 3);
			ConditionalsTable.Attach (ctl.RemoveButton, 2, 3, LastConditional - 1, LastConditional, AttachOptions.Shrink, AttachOptions.Fill, 3, 3);
		}
		
		
		protected void on_RemoveButton_clicked (object o, EventArgs args)
		{
			Gtk.Button button = o as Gtk.Button;
			
			foreach (Conditional ctl in Conditionals)
				if (ctl.RemoveButton == button) {
					RemoveConditional (ctl);
					return;
				}
		}
		
		
		protected void RemoveConditional (Conditional ctl)
		{
			ConditionalsTable.Remove (ctl.QueryType);
			ConditionalsTable.Remove (ctl.QueryWidget);
			ConditionalsTable.Remove (ctl.RemoveButton);
			Conditionals.Remove (ctl);
		}
	}
}
